import com.atlassian.jira.bc.security.login.LoginService
import com.atlassian.jira.component.ComponentAccessor
import com.atlassian.jira.security.roles.ProjectRoleManager

filename = "${title} - users.xlsx".toString() // "filename" is a standard Java String, not a GString

userExport = new UserExportTool()

/**
 * Velocity tool to export JIRA user account data.
 */
public class UserExportTool {
	def userManager = ComponentAccessor.userManager
	def groupManager = ComponentAccessor.groupManager
	def projectManager = ComponentAccessor.projectManager
	def projectRoleManager = ComponentAccessor.getOSGiComponentInstanceOfType(ProjectRoleManager.class)
	def loginService = ComponentAccessor.getComponent(LoginService.class)

	public getAllUsers() {
		def users = userManager.allApplicationUsers
		return users
	}

	public getStatus(String username) {
		def user = userManager.getUserByName(username)
		return user.isActive() ? "Active" : "Inactive"
	}

	public getDirectory(String username) {
		def user = userManager.getUserByName(username)
		return userManager.getDirectory(user.directoryId).name
	}

	public getGroups(String username) {
		def groups = groupManager.getGroupNamesForUser(username)
		return groups.join(", ")
	}

	public getLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.getLoginCount()
	}

	public getLastLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.lastLoginTime)
	}

	public getDaysSinceLastLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		def lastLoginDate = loginInfo?.lastLoginTime
		return (lastLoginDate != null) ? millisToDays(System.currentTimeMillis() - lastLoginDate) : null
	}

	public getPreviousLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.previousLoginTime)
	}

	public getLastFailedLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.lastFailedLoginTime)
	}

	public getCurrentFailedLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.currentFailedLoginCount
	}

	public getTotalFailedLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.totalFailedLoginCount // JIRA may return NULL for this even if "current failed login count" is 0
	}

	public getProjectRoles(String username) {
		def user = userManager.getUserByName(username)
		def projects = projectManager.projectObjects

		def rolesByProject = [:]
		projects.each {
			def project = it
			def roles = projectRoleManager.getProjectRoles(user, project)

			if(!roles.empty) {
				rolesByProject.put(project.key, roles.sort().join(", "))
			}
		}

		def roles = ""
		rolesByProject.sort().each { k, v ->
			roles += k + "=" + v + "; "
		}

		return roles
	}

	private unixTimeToExcelTime(millis) {
		return (millis != null) ? (25569 + millis/(24*60*60*1000)) : null
	}

	private millisToDays(millis) {
		return (millis != null) ? millis/(24*60*60*1000) : null
	}
}
